@@ -7,9 +7,10 @@
 # column rank predictor matrix. The solution is piecewise constant
 # over the grid.
 
-fusedlasso2d <- function(y, X, dim1, dim2, gamma=0, approx=FALSE, 
-                         maxsteps=2000, minlam=0, tol=1e-11, verbose=FALSE,
-                         fileback=FALSE) {
+fusedlasso2d <- function(y, X, dim1, dim2, gamma=0, approx=FALSE,
+                         maxsteps=2000, minlam=0, rtol=1e-7, btol=1e-7,
+                         eps=1e-4, verbose=FALSE) {
+
   if (missing(y)) stop("y is missing.")
   if (!is.numeric(y)) stop("y must be numeric.")
   if (length(y) == 0) stop("There must be at least one data point [must have length(y) > 1].")
@@ -34,9 +35,8 @@ fusedlasso2d <- function(y, X, dim1, dim2, gamma=0, approx=FALSE,
   }
 
   D = getD2dSparse(dim1,dim2)
-  out = fusedlasso(y,X,D,NULL,gamma,approx,maxsteps,minlam,tol,verbose,fileback)
+  out = fusedlasso(y,X,D,NULL,gamma,approx,maxsteps,minlam,rtol,btol,eps,verbose)
   out$call = match.call()
-  
-  if (fileback==FALSE) return(out)
-  else invisible(out)
+
+  return(out)
 }
